# NAME:     DQM_Copy_Results_with_Potential_Errors.py
# PURPOSE:   Parse the "PotentialErrorFile.txt" results from DQM, identify the "testDQM" files of interest,
# and copy those files into a new folder for further analysis.
# Developer: Barry Miller, Elevation Applied Researcher
# Organization: National Geospatial Technical Operations Center, United States Geological Survey
# Contact Information: bymiller@usgs.gov, (303) 202-4015
################################################################################

print "The python script is starting..."
import os,re
import shutil
import distutils.dir_util

################################################################################
# You only need to change these two variables to run the script.

#Identify where your DQM results are stored.
inputTextFileDirectory = r'I:\USGS_EDIT\FL_MiamiDade_B1_2015\metadata\ngtoc_created_metadata\ngtoc_edits\misc\DQM'
# Choose a destination folder. The potential error "testDQM" check files will be copied into this folder.
outputDQM_PotentialErrorDirectory = r'I:\USGS_EDIT\FL_MiamiDade_B1_2015\metadata\ngtoc_created_metadata\ngtoc_edits\misc\DQM\DQM_Files'
################################################################################

# Create list of all DQM test files and locate the potential error text file.
textFileList = []
potentialError_file = ""

for f in os.listdir(inputTextFileDirectory):
    # Add the "testDQM" check files into the "textFileList."
    if f.startswith("check") and f.endswith(".txt"):
    # Change non-alphanumeric character in the textfiledirectory to an underscore character : '_'
        #os.rename(os.path.join(inputTextFileDirectory, f), os.path.join(inputTextFileDirectory, f.replace("-", "_")))
        textFileList.append(os.path.join(inputTextFileDirectory, f))
    # Identify the location of the "PotentialErrorFile.txt."
    if f.startswith("Potential") and f.endswith(".txt"):
        potentialError_file = (os.path.join(inputTextFileDirectory, f))

if len(textFileList) == 0:
    print "There were no 'check' files found, please double check your input folder."
else:
    print str(len(textFileList)) + " check files were found in the input directory"

if potentialError_file == "":
    print "There was no 'PotentialError' file found, please double check your input folder."

else:
    print "Your Potential Error File is located at: " + potentialError_file


# Open the Potential Error file and copy the names of the "testDQM" check files of interest.
DQM_TestFiles_of_Interest = []

with open(potentialError_file) as f:
    # Remove white space in the text document.
    lines = [line.rstrip('\n') for line in f]
    # Skip the first line which just has the column headings.
    line_number = 0
    for line in lines:
        if line_number < 1:
            line_number += 1
        # Write the names of the Potential Error file in each line into the DQM_TestFiles_of_Interest list.
        else:
        # Rename all Potential Error files with alphanumeric characters    
            #original_name = open(potentialError_file).read()
            #new_name =re.sub('-', '_', original_name)
            #open(potentialError_file, 'w').write(new_name)
            
            splitLines = line.split()
            DQM_TestFiles_of_Interest.append(splitLines[0][0:-4]+"_" + splitLines[1]+ "_"+ splitLines[2]+".txt")
    print DQM_TestFiles_of_Interest

print "There are "+ str(len(DQM_TestFiles_of_Interest)) + " potential error testDQM files"


# Copy the "testDQM" check files with potential errors into a new directory.
try:
    print "The following testDQM check files will be copied to a new directory: "
    # Loop through each "testDQM" file and check to see if its name is in the "DQM_TestFiles_of_Interest" list.
    if len(textFileList) > 0:
        copyCounter = 0
        for textFile in textFileList:
            i = 0
            while i < len(DQM_TestFiles_of_Interest):
                if DQM_TestFiles_of_Interest[i] in textFile:
                    # Copy the "testDQM" file of interest into the output folder.
                    shutil.copy2(textFile, outputDQM_PotentialErrorDirectory + "\\" + DQM_TestFiles_of_Interest[i])
                    print textFile
                    copyCounter += 1
                i += 1
    else:
        print"Nothing could be copied, there are no 'testDQM' files"

    print "Congrats, " + str(copyCounter) +" testDQM check files were successfully copied to your output directory!"

except:
    pass
