# NAME:     DQM_Results_Txt_to_Shapefile.py
# PURPOSE:   Convert all "testDQM" check files generated by DQM and located in one folder into shapefiles for visualization.
# You will typically want to run this script after executing "DQM_Copy_Results_with_Potential_Errors.py"
# Developer: Barry Miller, Elevation Applied Researcher
# Organization: National Geospatial Technical Operations Center, United States Geological Survey
# Contact Information: bymiller@usgs.gov, (303) 202-4015
################################################################################

print "The python script is starting..."
import os
import arcpy


################################################################################
# You only need to change this variable to run the script.
#Identify where your DQM results of interest are stored.
textFileDirectory = r'I:\USGS_EDIT\FL_MiamiDade_B1_2015\metadata\ngtoc_created_metadata\ngtoc_edits\misc\DQM\DQM_Files'
################################################################################




# This is the list of fields within the "testDQM" file.
listOfFields = ['name', 'x', 'y', 'z', 'Nx', 'Ny', 'Nz', 'D', 'L1', 'L2', 'L3', 'dVx', 'dVy', 'dVz', 'distance', 'n_neighbor', 'Flag']
# This is the list of fields that will be populated in the shapefile.
listOfFields_with_Shape = ['name', 'x', 'y', 'z', 'Nx', 'Ny', 'Nz', 'D', 'L1', 'L2', 'L3', 'dVx', 'dVy', 'dVz', 'distance', 'n_neighbor', 'Flag', 'SHAPE@']

# Create list of text files in the input folder.
textFileList = []
for f in os.listdir(textFileDirectory):
    if f.startswith("check") and f.endswith(".txt"):
        textFileList.append(os.path.join(textFileDirectory, f))

if len(textFileList) == 0:
    print "There were no 'check' files found, please double check your input folder."
else:
    print str(len(textFileList)) + " check files were found in the input directory"

# Loop through all the check files and create shapefile for each text file.
try:
    shapefileCounter = 0
    for textFile in textFileList:
        # Assign a name for the shapefile based on the check file name.
        shapeFileName = os.path.basename(textFile).split(os.path.extsep)[0] + ".shp"
        # Determine the full file path of the future shapefile.
        fullShapeFileName = textFile.split(os.path.extsep)[0] + ".shp"
        # Create check file name attribute for list of check files.
        shapeFileName_Field = os.path.basename(textFile).split(os.path.extsep)[0]

        # Create an empty shapefile.
        if not arcpy.Exists(fullShapeFileName):
            try:
                arcpy.CreateFeatureclass_management(textFileDirectory, shapeFileName, "POINT")
            except:
                print (arcpy.GetMessages())

        # Check if the empty shapefile exists.
        if arcpy.Exists(fullShapeFileName):

            # Add the appropriate fields to the empty shapefile.
            for field in listOfFields:
                try:
                    if field == listOfFields[0]:
                        arcpy.AddField_management(fullShapeFileName, field, "TEXT")
                    else: 
                        arcpy.AddField_management(fullShapeFileName, field, "FLOAT")
                except:
                  print (arcpy.GetMessages())  
        

        # Read the current check file into a list of lines.
        with open(textFile) as f:
            listOfLines = []
            for line in f:
                trimmedline = line.replace("\n", "")
                listOfLines.append(trimmedline.split(" "))

        # Create an insert cursor for the current shapefile.
        print "These are all of the fields that will be populated in the shapefile:"
        print listOfFields_with_Shape

        cursor = arcpy.da.InsertCursor(fullShapeFileName, listOfFields_with_Shape)

        # Loop through each line in the current check file
        line_number = 0

        # Skip the first two lines which have the file locations and column headers.
        for line in listOfLines:
            if line_number < 2:
                line_number += 1

            # Assign the check file text document attributes to each column for the current row.
            else:

                row_name = str(shapeFileName_Field) 
                row_x = float(line[0])
                row_y = float(line[1])
                row_z = float(line[2])
                row_Nx = float(line[3])
                row_Ny = float(line[4])
                row_Nz = float(line[5])
                row_D = float(line[6])
                row_L1 = float(line[7])
                row_L2 = float(line[8])
                row_L3 = float(line[9])
                row_dVx = float(line[10])
                row_dVy = float(line[11])
                row_dVz = float(line[12])
                row_distance = float(line[13])
                row_n_neighbor = float(line[14])
                row_flag = float(line[15])

                # Write one line of check file data to the shapefile.
                newRow = (row_name, row_x, row_y, row_z, row_Nx, row_Ny, row_Nz, row_D, row_L1, row_L2, row_L3, row_dVx, row_dVy, row_dVz, row_distance, row_n_neighbor, row_flag, (row_x, row_y))
                cursor.insertRow(newRow)

        # Delete the cursor object.
        del cursor

        shapefileCounter += 1
        print "Finished converting", str(shapefileCounter), "of", str(len(textFileList))

    print "Congrats, " + str(shapefileCounter) +" check files were successfully converted into shapefiles!"

except:
    pass

# Create List of Shapefiles in the input folder.
ShapefileList = []
for f in os.listdir(textFileDirectory):
    if f.startswith("check") and f.endswith(".shp"):
        ShapefileList.append(os.path.join(textFileDirectory, f))
        
# Merge all check file shapefiles into one shapefile.
outputData = textFileDirectory + '\Merge_CheckFiles'
arcpy.Merge_management(ShapefileList, outputData)
print 'The shapefiles were SUCCESSFULLY merged into one shapefile: Merge_CheckFiles.shp' 

# Select Flag = 1 features from the Merge_CheckFiles shapefile.
where = '"FLAG" = 1'
arcpy.Select_analysis(textFileDirectory + '\Merge_CheckFiles.shp', textFileDirectory + '\Flag_1_CheckFiles.shp', where)
print (arcpy.GetMessages())
print 'Congrats! Export selected features Complete.'
