# NAME:     ProcDQMOutput.py DQMOutputAnalysis.py
# PURPOSE:   Model boresight errors in Lidar data swaths
# Developer: Ajit Sampath, Remote Sensing technologies Project, SGT, Contracter to USGS EROS
# PROJECT:   RST, for ASPRS Guidelines on Geometric accuracy of Lidar data
# Libraries: Standard python libraries plus numpy, scipy, matplotlib, liblas
# Version: 0.99a
# Revision History
# V0.99a: Ajit Sampath
#    Module Created
# V0.99b: SK Park   07/22/2019
#    Add new GUI and update fProcessTiles()
################################################################################
import sys
sys.path.append(r'C:\Python27\ArcGIS10.2\Lib\site-packages')
sys.path.append(r'C:\Python27\ArcGIS10.3\Lib\site-packages')
sys.path.append(r'C:\Python27\ArcGIS10.5\Lib\site-packages')
sys.path.append(r'C:\Python27\ArcGIS10.7\Lib\site-packages')
sys.path.append(r'C:\Python27\ArcGIS10.8\Lib\site-packages')
sys.path.append(r'C:\Python27\ArcGISx6410.8\Lib\site-packages')

#import os,numpy,random,time,math,subprocess, multiprocessing,struct
from Tkinter import *
import tkFileDialog
from DQMAnalysisFunctionsV2 import *
from mainframe_dqm import *

def fProcessTiles(DNAMES, data_type, ncores, n_points, n_distance, n_comparison, drop_class, drop_extended_class):
    DirectoryPath=DNAMES[0]
    OutputDirectorypath=DNAMES[1]
    os.chdir(OutputDirectorypath)
    
    listOfDirs = list()
    files = os.listdir(DirectoryPath)
    for f in files:
        if f.endswith('.laz') or f.endswith('.las') or f.endswith('.LAS'):
            listOfDirs += [DirectoryPath]
            break
    for (dirpath, dirnames, _) in os.walk(DirectoryPath):
        for dirname in dirnames:
            files = os.listdir(os.path.join(dirpath, dirname))
            for f in files:
                if f.endswith('.laz') or f.endswith('.las') or f.endswith('.LAS'):
                    listOfDirs += [os.path.join(dirpath, dirname)]
                    break
                            
    rdqmfname='runDQMnew'+'.bat'
    OFile= open(rdqmfname,'w+')    
    #OFile.write('set path= %Path%;Z:/research/BarryMiller/DQM_Testing;Z:/support/QC/LiDAR_QA_Support/Lidar_QA_Tools/LAStools/bin')
    # Add path to locate DQManage and DQMeasure
    OFile.write('set path= %Path%;'+os.path.dirname(sys.argv[0]))
    for dirname in listOfDirs:
        files = os.listdir(dirname)
        num_laz=0
        num_las=0
        for f in files:
            if f.endswith('.laz'):
                num_laz = num_laz+1
            if f.endswith('.las'):
                num_las = num_las+1
        cmd_args = ' -step 15 -eigen_ratio .005 -keep_single -drop_withheld ' + data_type + ' -max_number ' + str(n_comparison) +' -cores ' + str(ncores)
        if n_points > 0:
            cmd_args = cmd_args + ' -nearest ' + str(n_points)
        else:
            cmd_args = cmd_args + ' -distance ' + str(n_distance)            
        if drop_class != '':
            cmd_args = cmd_args + ' -drop_class ' + drop_class
        if drop_extended_class != '':
            cmd_args = cmd_args + ' -drop_extended_class ' + drop_extended_class   
        if num_las > 0:
            cm2="\nDQManage -i "+ '"'+ dirname +'/*.las"' + cmd_args
            OFile.write(cm2+'\n')
        if num_laz > 0:                
            cm2="\nDQManage -i "+ '"'+ dirname +'/*.laz"' + cmd_args       
            OFile.write(cm2+'\n')       
    
    OFile.close()
    os.system(rdqmfname)

def fCleanupAllBatchFiles(OutputDirectorypath):
    os.chdir(OutputDirectorypath)
    #os.system('del runDQM*.bat')

class App:
    LAS_dir_path=[]
    DQM_Output_path=[]

    def __init__(self, master):

        frame = Frame(master)
        frame.pack()
          
        self.chose_LAS_dir=Button(frame, text="Choose Directory of LAS Files", command= self.chose_LAS_dir)
        self.chose_LAS_dir.pack(side=LEFT)

        self.chose_DQM_Outputfolder=Button(frame, text="Choose Directory for DQM Output", command= self.chose_DQM_dir)
        self.chose_DQM_Outputfolder.pack(side=LEFT)

        self.button = Button(
            frame, text="Run DQM", fg="red", command=frame.quit
            )
        self.button.pack(side=LEFT)

        self.hi_there = Button(frame, text="Hello", command=self.say_hi)
        self.hi_there.pack(side=LEFT)

    def say_hi(self):
        print "hi there, everyone!"
        
    def chose_LAS_dir(self):
        self.LAS_dir_path = tkFileDialog.askdirectory(parent=root,initialdir="/",title='Please select a directory')
        if len(self.LAS_dir_path ) > 0:
            print "You chose %s" % self.LAS_dir_path

    def chose_DQM_dir(self):
        self.DQM_Output_path = tkFileDialog.askdirectory(parent=root,initialdir="/",title='Please select a directory')
        if len(self.DQM_Output_path ) > 0:
            print "You chose %s" % self.DQM_Output_path

###### MAIN PROGRAM#################################

if __name__ == '__main__':
    
    MIN_PTS=10
    data_type='-tiles'
    n_comparison=5000
    n_points=25    
    sw_version=2
    n_distance=0
    drop_class=''
    drop_extended_class=''
    h_error=0.0
    v_error=0.0
    angle=0.0
    
    if sw_version is 1:
        DirectoryPath=r'C:\Users\asampath\Desktop\USGS_LPC_PA_Allentown_2016_455903CA_LAS_2017'
        OutputDirectorypath=r'C:\Users\asampath\Desktop\USGS_LPC_PA_Allentown_2016_455903CA_LAS_2017'
        UNITS_FACTOR=1 # Change to .3048 for feet and 1 for meter
        MIN_PTS=10
    
        n_proc=multiprocessing.cpu_count()
        if(n_proc>10):
            n_proc=10
        root = Tk()
        app = App(root)
        root.mainloop()
        print app.LAS_dir_path, app.DQM_Output_path
    
        if(len(app.LAS_dir_path)>0):
            DirectoryPath=app.LAS_dir_path
            OutputDirectorypath=DirectoryPath
    
        if(len(app.DQM_Output_path)>0):
            OutputDirectorypath=app.DQM_Output_path
        root.destroy() # optional; see description below
        app=[]
        #app2=App2()
        #app2.fGetDirectoryNames(DirectoryPath,OutputDirectorypath)
    else:   
        root = Tk()
        root.title("Eros Lidar DQM")
            
        try:
            img = PhotoImage(file='image\usgs_logo.gif')
            root.tk.call('wm', 'iconphoto', root._w, img)
        except:
            print 'EROS logo icon not found!'        
        app = MainFrame(root)
        app.mainloop()    
        if app.okPressed is 0:
            print 'DQM is canceled'
            sys.exit(0)
        
        DirectoryPath=app.getInputPath()
        OutputDirectorypath=app.getOutputPath()
        UNITS_FACTOR=1 # Change to .3048 for feet and 1 for meter
        if app.getUnitOfMeasurement() is 1:        
            UNITS_FACTOR=0.3048
        n_proc = int(app.getNumCPUcores())
        neigboringParam = app.getNeighboringParam()
        if neigboringParam is 2:
            n_points = app.getNeighborPoints()
            n_distance = 0
        else:
            n_distance = float(app.getNeighborDistance())
            n_points = 0
        n_comparison = app.getNumComparison()
        h_error = float(app.getHErrorThreshold())
        v_error = float(app.getVErrorThreshold())
        angle = float(app.getDiscrepancyAngle()) 
        if app.getDataType() is 1:
            data_type = ''
        drop_class = app.getDropClassCode()
        drop_extended_class = app.getDropExtendedClassCode()
        root.destroy()
                
    fProcessTiles([DirectoryPath,OutputDirectorypath],data_type,n_proc,n_points,n_distance,n_comparison,drop_class,drop_extended_class)
    fGenerateAnalysis(OutputDirectorypath, UNITS_FACTOR,MIN_PTS,h_error,v_error,angle)
    fCleanupAllBatchFiles(OutputDirectorypath)

'''
##############################################################
## USER INPUTS
UNITS_FACTOR=1 # Change to .3048 for feet and 1 for meter
#DataDirectoryPath=r'D:\Data\CO'
#OutputDirectorypath=r'D:\Data\CO'
'''
